@@ -11,6 +11,7 @@ import (
 	"flag"
 	"fmt"
 	"math/rand"
+	"net/http"
 	"strings"
 	"sync"
 	"time"
@@ -20,6 +21,7 @@ import (
 	"github.com/gogo/protobuf/proto"
 	"github.com/grafana/dskit/kv"
 	"github.com/grafana/dskit/kv/codec"
+	"github.com/grafana/dskit/kv/memberlist"
 	"github.com/grafana/dskit/services"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promauto"
@@ -41,6 +43,14 @@ type haTrackerLimits interface {
 	MaxHAClusters(user string) int
 }
 
+type haTracker interface {
+	services.Service
+	http.Handler
+
+	checkReplica(ctx context.Context, userID, cluster, replica string, now time.Time) error
+	cleanupHATrackerMetricsForUser(userID string)
+}
+
 // ProtoReplicaDescFactory makes new InstanceDescs
 func ProtoReplicaDescFactory() proto.Message {
 	return NewReplicaDesc()
@@ -51,6 +61,77 @@ func NewReplicaDesc() *ReplicaDesc {
 	return &ReplicaDesc{}
 }
 
+// Merge merges other ReplicaDesc into this one.
+// The decision is made based on the ReceivedAt timestamp, if the Replica name is the same and at the ElectedAt if the
+// Replica name is different
+func (r *ReplicaDesc) Merge(other memberlist.Mergeable, _ bool) (change memberlist.Mergeable, error error) {
+	return r.mergeWithTime(other)
+}
+
+func (r *ReplicaDesc) mergeWithTime(mergeable memberlist.Mergeable) (memberlist.Mergeable, error) {
+	if mergeable == nil {
+		return nil, nil
+	}
+
+	other, ok := mergeable.(*ReplicaDesc)
+	if !ok {
+		return nil, fmt.Errorf("expected *distributor.ReplicaDesc, got %T", mergeable)
+	}
+
+	if other == nil {
+		return nil, nil
+	}
+
+	if other.Replica == r.Replica {
+		// Keeping the one with the most recent receivedAt timestamp
+		if other.ReceivedAt > r.ReceivedAt {
+			*r = *other
+		} else if r.ReceivedAt == other.ReceivedAt && r.DeletedAt == 0 && other.DeletedAt != 0 {
+			*r = *other
+		}
+	} else {
+		// keep the most recent ElectedAt to reach consistency
+		if other.ElectedAt > r.ElectedAt {
+			*r = *other
+		} else if other.ElectedAt == r.ElectedAt {
+			// if the timestamps are equal we compare ReceivedAt
+			if other.ReceivedAt > r.ReceivedAt {
+				*r = *other
+			}
+		}
+	}
+
+	// No changes
+	if *r != *other {
+		return nil, nil
+	}
+
+	out := NewReplicaDesc()
+	*out = *r
+	return out, nil
+}
+
+// MergeContent describes content of this Mergeable.
+// Given that ReplicaDesc can have only one instance at a time, it returns the ReplicaDesc it contains. By doing this we choose
+// to not make use of the subset invalidation feature of memberlist
+func (r *ReplicaDesc) MergeContent() []string {
+	result := []string(nil)
+	if len(r.Replica) != 0 {
+		result = append(result, r.String())
+	}
+	return result
+}
+
+// RemoveTombstones is noOp because we will handle replica deletions outside the context of memberlist.
+func (r *ReplicaDesc) RemoveTombstones(_ time.Time) (total, removed int) {
+	return
+}
+
+// Clone returns a deep copy of the ReplicaDesc.
+func (r *ReplicaDesc) Clone() memberlist.Mergeable {
+	return proto.Clone(r).(*ReplicaDesc)
+}
+
 // HATrackerConfig contains the configuration required to
 // create an HA Tracker.
 type HATrackerConfig struct {
@@ -107,7 +188,7 @@ func GetReplicaDescCodec() codec.Proto {
 
 // Track the replica we're accepting samples from
 // for each HA cluster we know about.
-type haTracker struct {
+type defaultHaTracker struct {
 	services.Service
 
 	logger              log.Logger
@@ -140,15 +221,15 @@ type haClusterInfo struct {
 	nonElectedLastSeenTimestamp int64 // timestamp in milliseconds
 }
 
-// newHATracker returns a new HA cluster tracker using either Consul,
+// newHaTracker returns a new HA cluster tracker using either Consul,
 // etcd, or an in-memory KV store. Tracker must be started via StartAsync().
-func newHATracker(cfg HATrackerConfig, limits haTrackerLimits, reg prometheus.Registerer, logger log.Logger) (*haTracker, error) {
+func newHaTracker(cfg HATrackerConfig, limits haTrackerLimits, reg prometheus.Registerer, logger log.Logger) (*defaultHaTracker, error) {
 	var jitter time.Duration
 	if cfg.UpdateTimeoutJitterMax > 0 {
 		jitter = time.Duration(rand.Int63n(int64(2*cfg.UpdateTimeoutJitterMax))) - cfg.UpdateTimeoutJitterMax
 	}
 
-	t := &haTracker{
+	t := &defaultHaTracker{
 		logger:              log.With(logger, "component", "ha-tracker"),
 		cfg:                 cfg,
 		updateTimeoutJitter: jitter,
@@ -199,31 +280,58 @@ func newHATracker(cfg HATrackerConfig, limits haTrackerLimits, reg prometheus.Re
 		}),
 	}
 
-	if cfg.EnableHATracker {
-		client, err := kv.NewClient(
-			cfg.KVStore,
-			GetReplicaDescCodec(),
-			kv.RegistererWithKVName(prometheus.WrapRegistererWithPrefix("cortex_", reg), "distributor-hatracker"),
-			logger,
-		)
-		if err != nil {
-			return nil, err
-		}
-		t.client = client
+	client, err := kv.NewClient(
+		cfg.KVStore,
+		GetReplicaDescCodec(),
+		kv.RegistererWithKVName(prometheus.WrapRegistererWithPrefix("cortex_", reg), "distributor-hatracker"),
+		logger,
+	)
+	if err != nil {
+		return nil, err
 	}
+	t.client = client
 
-	t.Service = services.NewBasicService(nil, t.loop, nil)
+	t.Service = services.NewBasicService(t.syncHATrackerStateOnStart, t.loop, nil)
 	return t, nil
 }
 
-// Follows pattern used by ring for WatchKey.
-func (h *haTracker) loop(ctx context.Context) error {
-	if !h.cfg.EnableHATracker {
-		// don't do anything, but wait until asked to stop.
-		<-ctx.Done()
+func (h *defaultHaTracker) syncHATrackerStateOnStart(ctx context.Context) error {
+	level.Info(h.logger).Log("msg", "sync HA state on start: Listing keys from KV Store")
+	keys, err := h.client.List(ctx, "")
+	if err != nil {
+		level.Error(h.logger).Log("msg", "sync HA state on start: failed to list the keys ", "err", err)
+		return err
+	}
+
+	if len(keys) == 0 {
+		level.Warn(h.logger).Log("msg", "sync HA state on start: no keys for HA tracker prefix in the KV store."+
+			"Skipping cache sync")
 		return nil
 	}
 
+	for i := 0; i < len(keys); i++ {
+		if ctx.Err() != nil {
+			return fmt.Errorf("syncing HA tracker state on startup: %w", context.Cause(ctx))
+		}
+
+		val, err := h.client.Get(ctx, keys[i])
+		if err != nil {
+			level.Warn(h.logger).Log("msg", "sync HA state on start: failed to get replica value", "key", keys[i], "err", err)
+			return nil
+		}
+		desc, ok := val.(*ReplicaDesc)
+		if !ok {
+			level.Error(h.logger).Log("msg", "sync HA state on start: Skipping key: "+keys[i]+", got invalid ReplicaDesc")
+			continue
+		}
+		h.processKVStoreEntry(keys[i], desc)
+	}
+	level.Info(h.logger).Log("msg", "sync HA state on start: HA cache sync finished successfully")
+	return nil
+}
+
+// Follows pattern used by ring for WatchKey.
+func (h *defaultHaTracker) loop(ctx context.Context) error {
 	// Start cleanup loop. It will stop when context is done.
 	wg := sync.WaitGroup{}
 	wg.Add(1)
@@ -236,40 +344,11 @@ func (h *haTracker) loop(ctx context.Context) error {
 	// The KVStore config we gave when creating h should have contained a prefix,
 	// which would have given us a prefixed KVStore client. So, we can pass an empty string here.
 	h.client.WatchPrefix(ctx, "", func(key string, value interface{}) bool {
-		replica := value.(*ReplicaDesc)
-		segments := strings.SplitN(key, "/", 2)
-
-		// Valid key would look like cluster/replica, and a key without a / such as `ring` would be invalid.
-		if len(segments) != 2 {
-			return true
+		replica, ok := value.(*ReplicaDesc)
+		if !ok {
+			return false
 		}
-
-		user := segments[0]
-		cluster := segments[1]
-
-		if replica.DeletedAt > 0 {
-			h.electedReplicaChanges.DeleteLabelValues(user, cluster)
-			h.electedReplicaTimestamp.DeleteLabelValues(user, cluster)
-			h.lastElectionTimestamp.DeleteLabelValues(user, cluster)
-			h.totalReelections.DeleteLabelValues(user, cluster)
-
-			h.electedLock.Lock()
-			defer h.electedLock.Unlock()
-			userClusters := h.clusters[user]
-			if userClusters != nil {
-				delete(userClusters, cluster)
-				if len(userClusters) == 0 {
-					delete(h.clusters, user)
-				}
-			}
-			return true
-		}
-
-		// Store the received information into our cache
-		h.electedLock.Lock()
-		h.updateCache(user, cluster, replica)
-		h.electedLock.Unlock()
-		h.electedReplicaPropagationTime.Observe(time.Since(timestamp.Time(replica.ReceivedAt)).Seconds())
+		h.processKVStoreEntry(key, replica)
 		return true
 	})
 
@@ -277,6 +356,46 @@ func (h *haTracker) loop(ctx context.Context) error {
 	return nil
 }
 
+func (h *defaultHaTracker) processKVStoreEntry(key string, replica *ReplicaDesc) {
+	segments := strings.SplitN(key, "/", 2)
+
+	// Valid key would look like cluster/replica, and a key without a / such as `ring` would be invalid.
+	if len(segments) != 2 {
+		return
+	}
+
+	user := segments[0]
+	cluster := segments[1]
+
+	if replica.DeletedAt > 0 {
+		h.cleanupDeletedReplica(user, cluster)
+		return
+	}
+
+	// Store the received information into our cache
+	h.electedLock.Lock()
+	h.updateCache(user, cluster, replica)
+	h.electedLock.Unlock()
+	h.electedReplicaPropagationTime.Observe(time.Since(timestamp.Time(replica.ReceivedAt)).Seconds())
+}
+
+func (h *defaultHaTracker) cleanupDeletedReplica(user string, cluster string) {
+	h.electedReplicaChanges.DeleteLabelValues(user, cluster)
+	h.electedReplicaTimestamp.DeleteLabelValues(user, cluster)
+	h.lastElectionTimestamp.DeleteLabelValues(user, cluster)
+	h.totalReelections.DeleteLabelValues(user, cluster)
+
+	h.electedLock.Lock()
+	defer h.electedLock.Unlock()
+	userClusters := h.clusters[user]
+	if userClusters != nil {
+		delete(userClusters, cluster)
+		if len(userClusters) == 0 {
+			delete(h.clusters, user)
+		}
+	}
+}
+
 const (
 	cleanupCyclePeriod         = 30 * time.Minute
 	cleanupCycleJitterVariance = 0.2 // for 30 minutes, this is ±6 min
@@ -286,7 +405,7 @@ const (
 	deletionTimeout = 30 * time.Minute
 )
 
-func (h *haTracker) updateKVLoop(ctx context.Context) {
+func (h *defaultHaTracker) updateKVLoop(ctx context.Context) {
 	cleanupTick := time.NewTicker(util.DurationWithJitter(cleanupCyclePeriod, cleanupCycleJitterVariance))
 	defer cleanupTick.Stop()
 	tick := time.NewTicker(h.cfg.UpdateTimeout)
@@ -307,7 +426,7 @@ func (h *haTracker) updateKVLoop(ctx context.Context) {
 
 // Loop over all entries in our cache and update KVStore where it is out of date,
 // electing a new replica if necessary.
-func (h *haTracker) updateKVStoreAll(ctx context.Context, now time.Time) {
+func (h *defaultHaTracker) updateKVStoreAll(ctx context.Context, now time.Time) {
 	h.electedLock.RLock()
 	defer h.electedLock.RUnlock()
 	// Note the maps may change when we release the lock while talking to KVStore;
@@ -344,7 +463,7 @@ func (h *haTracker) updateKVStoreAll(ctx context.Context, now time.Time) {
 
 // Replicas marked for deletion before deadline will be deleted.
 // Replicas with last-received timestamp before deadline will be marked for deletion.
-func (h *haTracker) cleanupOldReplicas(ctx context.Context, deadline time.Time) {
+func (h *defaultHaTracker) cleanupOldReplicas(ctx context.Context, deadline time.Time) {
 	keys, err := h.client.List(ctx, "")
 	if err != nil {
 		level.Warn(h.logger).Log("msg", "cleanup: failed to list replica keys", "err", err)
@@ -422,12 +541,7 @@ func (h *haTracker) cleanupOldReplicas(ctx context.Context, deadline time.Time)
 // Updates to and from the KV store are handled in the background, except
 // if we have no cached data for this cluster in which case we create the
 // record and store it in-band.
-func (h *haTracker) checkReplica(ctx context.Context, userID, cluster, replica string, now time.Time) error {
-	// If HA tracking isn't enabled then accept the sample
-	if !h.cfg.EnableHATracker {
-		return nil
-	}
-
+func (h *defaultHaTracker) checkReplica(ctx context.Context, userID, cluster, replica string, now time.Time) error {
 	h.electedLock.Lock()
 	if entry := h.clusters[userID][cluster]; entry != nil {
 		var err error
@@ -461,12 +575,12 @@ func (h *haTracker) checkReplica(ctx context.Context, userID, cluster, replica s
 	return h.checkReplica(ctx, userID, cluster, replica, now)
 }
 
-func (h *haTracker) withinUpdateTimeout(now time.Time, receivedAt int64) bool {
+func (h *defaultHaTracker) withinUpdateTimeout(now time.Time, receivedAt int64) bool {
 	return now.Sub(timestamp.Time(receivedAt)) < h.cfg.UpdateTimeout+h.updateTimeoutJitter
 }
 
 // Must be called with electedLock held.
-func (h *haTracker) updateCache(userID, cluster string, desc *ReplicaDesc) {
+func (h *defaultHaTracker) updateCache(userID, cluster string, desc *ReplicaDesc) {
 	if h.clusters[userID] == nil {
 		h.clusters[userID] = map[string]*haClusterInfo{}
 	}
@@ -493,7 +607,7 @@ func (h *haTracker) updateCache(userID, cluster string, desc *ReplicaDesc) {
 
 // If we do set the value then err will be nil and desc will contain the value we set.
 // If there is already a valid value in the store, return nil, nil.
-func (h *haTracker) updateKVStore(ctx context.Context, userID, cluster, replica string, now time.Time, receivedAt int64) error {
+func (h *defaultHaTracker) updateKVStore(ctx context.Context, userID, cluster, replica string, now time.Time, receivedAt int64) error {
 	key := fmt.Sprintf("%s/%s", userID, cluster)
 	var desc *ReplicaDesc
 	var electedAtTime, electedChanges int64
@@ -559,7 +673,7 @@ func findHALabels(replicaLabel, clusterLabel string, labels []mimirpb.LabelAdapt
 	return cluster, replica
 }
 
-func (h *haTracker) cleanupHATrackerMetricsForUser(userID string) {
+func (h *defaultHaTracker) cleanupHATrackerMetricsForUser(userID string) {
 	filter := prometheus.Labels{"user": userID}
 
 	h.electedReplicaChanges.DeletePartialMatch(filter)
