@@ -49,7 +49,7 @@ import (
 func TestHandler_remoteWrite(t *testing.T) {
 	req := createRequest(t, createPrometheusRemoteWriteProtobuf(t))
 	resp := httptest.NewRecorder()
-	handler := Handler(100000, nil, nil, false, validation.MockDefaultOverrides(), RetryConfig{}, verifyWritePushFunc(t, mimirpb.API), nil, log.NewNopLogger())
+	handler := Handler(100000, nil, nil, false, false, validation.MockDefaultOverrides(), RetryConfig{}, verifyWritePushFunc(t, mimirpb.API), nil, log.NewNopLogger())
 	handler.ServeHTTP(resp, req)
 	assert.Equal(t, 200, resp.Code)
 }
@@ -114,19 +114,19 @@ func TestOTelMetricsToMetadata(t *testing.T) {
 }
 
 func TestHandler_mimirWriteRequest(t *testing.T) {
-	req := createRequest(t, createMimirWriteRequestProtobuf(t, false))
+	req := createRequest(t, createMimirWriteRequestProtobuf(t, false, false))
 	resp := httptest.NewRecorder()
 	sourceIPs, _ := middleware.NewSourceIPs("SomeField", "(.*)", false)
-	handler := Handler(100000, nil, sourceIPs, false, validation.MockDefaultOverrides(), RetryConfig{}, verifyWritePushFunc(t, mimirpb.RULE), nil, log.NewNopLogger())
+	handler := Handler(100000, nil, sourceIPs, false, false, validation.MockDefaultOverrides(), RetryConfig{}, verifyWritePushFunc(t, mimirpb.RULE), nil, log.NewNopLogger())
 	handler.ServeHTTP(resp, req)
 	assert.Equal(t, 200, resp.Code)
 }
 
 func TestHandler_contextCanceledRequest(t *testing.T) {
-	req := createRequest(t, createMimirWriteRequestProtobuf(t, false))
+	req := createRequest(t, createMimirWriteRequestProtobuf(t, false, false))
 	resp := httptest.NewRecorder()
 	sourceIPs, _ := middleware.NewSourceIPs("SomeField", "(.*)", false)
-	handler := Handler(100000, nil, sourceIPs, false, validation.MockDefaultOverrides(), RetryConfig{}, func(_ context.Context, req *Request) error {
+	handler := Handler(100000, nil, sourceIPs, false, false, validation.MockDefaultOverrides(), RetryConfig{}, func(_ context.Context, req *Request) error {
 		defer req.CleanUp()
 		return fmt.Errorf("the request failed: %w", context.Canceled)
 	}, nil, log.NewNopLogger())
@@ -144,7 +144,7 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 		expectedStatusCode                        int
 	}{
 		{
-			name:                         "config flag set to false means SkipLabelNameValidation is false",
+			name:                         "config flag set to false means SkipLabelValidation is false",
 			allowSkipLabelNameValidation: false,
 			req:                          createRequest(t, createMimirWriteRequestProtobufWithNonSupportedLabelNames(t, false)),
 			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
@@ -154,7 +154,7 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 				assert.Equal(t, "a-label", request.Timeseries[0].Labels[0].Name)
 				assert.Equal(t, "value", request.Timeseries[0].Labels[0].Value)
 				assert.Equal(t, mimirpb.RULE, request.Source)
-				assert.False(t, request.SkipLabelNameValidation)
+				assert.False(t, request.SkipLabelValidation)
 				pushReq.CleanUp()
 				return nil
 			},
@@ -162,7 +162,7 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 			expectedStatusCode:                        http.StatusOK,
 		},
 		{
-			name:                         "config flag set to false means SkipLabelNameValidation is always false even if write requests sets it to true",
+			name:                         "config flag set to false means SkipLabelValidation is always false even if write requests sets it to true",
 			allowSkipLabelNameValidation: false,
 			req:                          createRequest(t, createMimirWriteRequestProtobufWithNonSupportedLabelNames(t, true)),
 			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
@@ -173,14 +173,14 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 				assert.Equal(t, "a-label", request.Timeseries[0].Labels[0].Name)
 				assert.Equal(t, "value", request.Timeseries[0].Labels[0].Value)
 				assert.Equal(t, mimirpb.RULE, request.Source)
-				assert.False(t, request.SkipLabelNameValidation)
+				assert.False(t, request.SkipLabelValidation)
 				return nil
 			},
 			includeAllowSkiplabelNameValidationHeader: true,
 			expectedStatusCode:                        http.StatusOK,
 		},
 		{
-			name:                         "config flag set to true but write request set to false means SkipLabelNameValidation is false",
+			name:                         "config flag set to true but write request set to false means SkipLabelValidation is false",
 			allowSkipLabelNameValidation: true,
 			req:                          createRequest(t, createMimirWriteRequestProtobufWithNonSupportedLabelNames(t, false)),
 			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
@@ -190,14 +190,14 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 				assert.Equal(t, "a-label", request.Timeseries[0].Labels[0].Name)
 				assert.Equal(t, "value", request.Timeseries[0].Labels[0].Value)
 				assert.Equal(t, mimirpb.RULE, request.Source)
-				assert.False(t, request.SkipLabelNameValidation)
+				assert.False(t, request.SkipLabelValidation)
 				pushReq.CleanUp()
 				return nil
 			},
 			expectedStatusCode: http.StatusOK,
 		},
 		{
-			name:                         "config flag set to true and write request set to true means SkipLabelNameValidation is true",
+			name:                         "config flag set to true and write request set to true means SkipLabelValidation is true",
 			allowSkipLabelNameValidation: true,
 			req:                          createRequest(t, createMimirWriteRequestProtobufWithNonSupportedLabelNames(t, true)),
 			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
@@ -207,14 +207,14 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 				assert.Equal(t, "a-label", request.Timeseries[0].Labels[0].Name)
 				assert.Equal(t, "value", request.Timeseries[0].Labels[0].Value)
 				assert.Equal(t, mimirpb.RULE, request.Source)
-				assert.True(t, request.SkipLabelNameValidation)
+				assert.True(t, request.SkipLabelValidation)
 				pushReq.CleanUp()
 				return nil
 			},
 			expectedStatusCode: http.StatusOK,
 		},
 		{
-			name:                         "config flag set to true and write request set to true but header not sent means SkipLabelNameValidation is false",
+			name:                         "config flag set to true and write request set to true but header not sent means SkipLabelValidation is false",
 			allowSkipLabelNameValidation: true,
 			req:                          createRequest(t, createMimirWriteRequestProtobufWithNonSupportedLabelNames(t, true)),
 			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
@@ -224,7 +224,7 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 				assert.Equal(t, "a-label", request.Timeseries[0].Labels[0].Name)
 				assert.Equal(t, "value", request.Timeseries[0].Labels[0].Value)
 				assert.Equal(t, mimirpb.RULE, request.Source)
-				assert.False(t, request.SkipLabelNameValidation)
+				assert.False(t, request.SkipLabelValidation)
 				pushReq.CleanUp()
 				return nil
 			},
@@ -235,7 +235,7 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 	for _, tc := range tests {
 		t.Run(tc.name, func(t *testing.T) {
 			resp := httptest.NewRecorder()
-			handler := Handler(100000, nil, nil, tc.allowSkipLabelNameValidation, validation.MockDefaultOverrides(), RetryConfig{}, tc.verifyReqHandler, nil, log.NewNopLogger())
+			handler := Handler(100000, nil, nil, tc.allowSkipLabelNameValidation, false, validation.MockDefaultOverrides(), RetryConfig{}, tc.verifyReqHandler, nil, log.NewNopLogger())
 			if !tc.includeAllowSkiplabelNameValidationHeader {
 				tc.req.Header.Set(SkipLabelNameValidationHeader, "true")
 			}
@@ -245,6 +245,108 @@ func TestHandler_EnsureSkipLabelNameValidationBehaviour(t *testing.T) {
 	}
 }
 
+func TestHandler_EnsureSkipLabelCountValidationBehaviour(t *testing.T) {
+	tests := []struct {
+		name                                       string
+		allowSkipLabelCountValidation              bool
+		includeAllowSkipLabelCountValidationHeader bool
+		req                                        *http.Request
+		verifyReqHandler                           PushFunc
+		expectedStatusCode                         int
+	}{
+		{
+			name:                          "config flag set to false means SkipLabelCountValidation is false",
+			allowSkipLabelCountValidation: false,
+			includeAllowSkipLabelCountValidationHeader: true,
+			req: createRequest(t, createMimirWriteRequestProtobuf(t, false, false)),
+			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
+				request, err := pushReq.WriteRequest()
+				assert.NoError(t, err)
+				assert.Len(t, request.Timeseries, 1)
+				assert.Equal(t, mimirpb.RULE, request.Source)
+				assert.False(t, request.SkipLabelCountValidation)
+				pushReq.CleanUp()
+				return nil
+			},
+			expectedStatusCode: http.StatusOK,
+		},
+		{
+			name:                          "config flag set to false means SkipLabelCountValidation is always false even if write requests sets it to true",
+			allowSkipLabelCountValidation: false,
+			includeAllowSkipLabelCountValidationHeader: true,
+			req: createRequest(t, createMimirWriteRequestProtobuf(t, false, true)),
+			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
+				request, err := pushReq.WriteRequest()
+				require.NoError(t, err)
+				t.Cleanup(pushReq.CleanUp)
+				assert.Len(t, request.Timeseries, 1)
+				assert.Equal(t, mimirpb.RULE, request.Source)
+				assert.False(t, request.SkipLabelCountValidation)
+				return nil
+			},
+			expectedStatusCode: http.StatusOK,
+		},
+		{
+			name:                          "config flag set to true but write request set to false means SkipLabelCountValidation is false",
+			allowSkipLabelCountValidation: true,
+			req:                           createRequest(t, createMimirWriteRequestProtobuf(t, false, false)),
+			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
+				request, err := pushReq.WriteRequest()
+				assert.NoError(t, err)
+				assert.Len(t, request.Timeseries, 1)
+				assert.Equal(t, mimirpb.RULE, request.Source)
+				assert.False(t, request.SkipLabelCountValidation)
+				pushReq.CleanUp()
+				return nil
+			},
+			expectedStatusCode: http.StatusOK,
+		},
+		{
+			name:                          "config flag set to true and write request set to true means SkipLabelCountValidation is true",
+			allowSkipLabelCountValidation: true,
+			req:                           createRequest(t, createMimirWriteRequestProtobuf(t, false, true)),
+			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
+				request, err := pushReq.WriteRequest()
+				assert.NoError(t, err)
+				assert.Len(t, request.Timeseries, 1)
+				assert.Equal(t, mimirpb.RULE, request.Source)
+				assert.True(t, request.SkipLabelCountValidation)
+				pushReq.CleanUp()
+				return nil
+			},
+			expectedStatusCode: http.StatusOK,
+		},
+		{
+			name:                          "config flag set to true and write request set to true but header not sent means SkipLabelCountValidation is false",
+			allowSkipLabelCountValidation: true,
+			includeAllowSkipLabelCountValidationHeader: true,
+			req: createRequest(t, createMimirWriteRequestProtobuf(t, false, true)),
+			verifyReqHandler: func(_ context.Context, pushReq *Request) error {
+				request, err := pushReq.WriteRequest()
+				assert.NoError(t, err)
+				assert.Len(t, request.Timeseries, 1)
+				assert.Equal(t, mimirpb.RULE, request.Source)
+				assert.False(t, request.SkipLabelCountValidation)
+				pushReq.CleanUp()
+				return nil
+			},
+			expectedStatusCode: http.StatusOK,
+		},
+	}
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			resp := httptest.NewRecorder()
+			limits := validation.MockDefaultOverrides()
+			handler := Handler(100000, nil, nil, false, tc.allowSkipLabelCountValidation, limits, RetryConfig{}, tc.verifyReqHandler, nil, log.NewNopLogger())
+			if !tc.includeAllowSkipLabelCountValidationHeader {
+				tc.req.Header.Set(SkipLabelCountValidationHeader, "true")
+			}
+			handler.ServeHTTP(resp, tc.req)
+			assert.Equal(t, tc.expectedStatusCode, resp.Code)
+		})
+	}
+}
+
 func TestHandler_SkipExemplarUnmarshalingBasedOnLimits(t *testing.T) {
 	timestampMs := time.Now().UnixMilli()
 
@@ -355,7 +457,7 @@ func TestHandler_SkipExemplarUnmarshalingBasedOnLimits(t *testing.T) {
 			require.NoError(t, err)
 
 			var gotReqEncoded *Request
-			handler := Handler(100000, nil, nil, true, limits, RetryConfig{}, func(_ context.Context, pushReq *Request) error {
+			handler := Handler(100000, nil, nil, true, false, limits, RetryConfig{}, func(_ context.Context, pushReq *Request) error {
 				gotReqEncoded = pushReq
 				return nil
 			}, nil, log.NewNopLogger())
@@ -390,7 +492,7 @@ func verifyWritePushFunc(t *testing.T, expectSource mimirpb.WriteRequest_SourceE
 		require.Equal(t, "__name__", request.Timeseries[0].Labels[0].Name)
 		require.Equal(t, "foo", request.Timeseries[0].Labels[0].Value)
 		require.Equal(t, expectSource, request.Source)
-		require.False(t, request.SkipLabelNameValidation)
+		require.False(t, request.SkipLabelValidation)
 		return nil
 	}
 }
@@ -441,7 +543,7 @@ func createPrometheusRemoteWriteProtobuf(t testing.TB) []byte {
 	return inputBytes
 }
 
-func createMimirWriteRequestProtobuf(t *testing.T, skipLabelNameValidation bool) []byte {
+func createMimirWriteRequestProtobuf(t *testing.T, skipLabelNameValidation, skipLabelCountValidation bool) []byte {
 	t.Helper()
 	h := prompb.FromIntHistogram(1337, test.GenerateTestHistogram(1))
 	ts := mimirpb.PreallocTimeseries{
@@ -456,9 +558,10 @@ func createMimirWriteRequestProtobuf(t *testing.T, skipLabelNameValidation bool)
 		},
 	}
 	input := mimirpb.WriteRequest{
-		Timeseries:              []mimirpb.PreallocTimeseries{ts},
-		Source:                  mimirpb.RULE,
-		SkipLabelNameValidation: skipLabelNameValidation,
+		Timeseries:               []mimirpb.PreallocTimeseries{ts},
+		Source:                   mimirpb.RULE,
+		SkipLabelValidation:      skipLabelNameValidation,
+		SkipLabelCountValidation: skipLabelCountValidation,
 	}
 	inoutBytes, err := input.Marshal()
 	require.NoError(t, err)
@@ -478,9 +581,9 @@ func createMimirWriteRequestProtobufWithNonSupportedLabelNames(t *testing.T, ski
 		},
 	}
 	input := mimirpb.WriteRequest{
-		Timeseries:              []mimirpb.PreallocTimeseries{ts},
-		Source:                  mimirpb.RULE,
-		SkipLabelNameValidation: skipLabelNameValidation,
+		Timeseries:          []mimirpb.PreallocTimeseries{ts},
+		Source:              mimirpb.RULE,
+		SkipLabelValidation: skipLabelNameValidation,
 	}
 	inoutBytes, err := input.Marshal()
 	require.NoError(t, err)
@@ -498,7 +601,7 @@ func BenchmarkPushHandler(b *testing.B) {
 		pushReq.CleanUp()
 		return nil
 	}
-	handler := Handler(100000, nil, nil, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, nil, log.NewNopLogger())
+	handler := Handler(100000, nil, nil, false, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, nil, log.NewNopLogger())
 	b.ResetTimer()
 	for iter := 0; iter < b.N; iter++ {
 		req.Body = bufCloser{Buffer: buf} // reset Body so it can be read each time round the loop
@@ -558,7 +661,7 @@ func TestHandler_ErrorTranslation(t *testing.T) {
 			}
 
 			logs := &concurrency.SyncBuffer{}
-			h := handler(10, nil, nil, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, log.NewLogfmtLogger(logs), parserFunc)
+			h := handler(10, nil, nil, false, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, log.NewLogfmtLogger(logs), parserFunc)
 
 			recorder := httptest.NewRecorder()
 			ctxWithUser := user.InjectOrgID(context.Background(), "testuser")
@@ -648,7 +751,7 @@ func TestHandler_ErrorTranslation(t *testing.T) {
 			}
 
 			logs := &concurrency.SyncBuffer{}
-			h := handler(10, nil, nil, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, log.NewLogfmtLogger(logs), parserFunc)
+			h := handler(10, nil, nil, false, false, validation.MockDefaultOverrides(), RetryConfig{}, pushFunc, log.NewLogfmtLogger(logs), parserFunc)
 			recorder := httptest.NewRecorder()
 			ctxWithUser := user.InjectOrgID(context.Background(), "testuser")
 			h.ServeHTTP(recorder, httptest.NewRequest(http.MethodPost, "/push", bufCloser{&bytes.Buffer{}}).WithContext(ctxWithUser))
@@ -1080,7 +1183,7 @@ func TestOTLPPushHandlerErrorsAreReportedCorrectlyViaHttpgrpc(t *testing.T) {
 
 		return nil
 	}
-	h := OTLPHandler(200, util.NewBufferPool(), nil, otlpLimitsMock{}, RetryConfig{}, push, newPushMetrics(reg), reg, log.NewNopLogger(), true)
+	h := OTLPHandler(200, util.NewBufferPool(0), nil, otlpLimitsMock{}, nil, RetryConfig{}, push, newPushMetrics(reg), reg, log.NewNopLogger())
 	srv.HTTP.Handle("/otlp", h)
 
 	// start the server
@@ -1244,10 +1347,51 @@ func mustMarshalStatus(t *testing.T, code codes.Code, msg string) []byte {
 
 type otlpLimitsMock struct{}
 
-func (o otlpLimitsMock) OTelMetricSuffixesEnabled(_ string) bool {
+func (o otlpLimitsMock) OTelMetricSuffixesEnabled(string) bool {
 	return false
 }
 
-func (o otlpLimitsMock) OTelCreatedTimestampZeroIngestionEnabled(_ string) bool {
+func (o otlpLimitsMock) OTelCreatedTimestampZeroIngestionEnabled(string) bool {
 	return false
 }
+
+func (o otlpLimitsMock) PromoteOTelResourceAttributes(string) []string {
+	return nil
+}
+
+func promToMimirHistogram(h *prompb.Histogram) mimirpb.Histogram {
+	pSpans := make([]mimirpb.BucketSpan, 0, len(h.PositiveSpans))
+	for _, span := range h.PositiveSpans {
+		pSpans = append(
+			pSpans, mimirpb.BucketSpan{
+				Offset: span.Offset,
+				Length: span.Length,
+			},
+		)
+	}
+	nSpans := make([]mimirpb.BucketSpan, 0, len(h.NegativeSpans))
+	for _, span := range h.NegativeSpans {
+		nSpans = append(
+			nSpans, mimirpb.BucketSpan{
+				Offset: span.Offset,
+				Length: span.Length,
+			},
+		)
+	}
+
+	return mimirpb.Histogram{
+		Count:          &mimirpb.Histogram_CountInt{CountInt: h.GetCountInt()},
+		Sum:            h.Sum,
+		Schema:         h.Schema,
+		ZeroThreshold:  h.ZeroThreshold,
+		ZeroCount:      &mimirpb.Histogram_ZeroCountInt{ZeroCountInt: h.GetZeroCountInt()},
+		NegativeSpans:  nSpans,
+		NegativeDeltas: h.NegativeDeltas,
+		NegativeCounts: h.NegativeCounts,
+		PositiveSpans:  pSpans,
+		PositiveDeltas: h.PositiveDeltas,
+		PositiveCounts: h.PositiveCounts,
+		Timestamp:      h.Timestamp,
+		ResetHint:      mimirpb.Histogram_ResetHint(h.ResetHint),
+	}
+}
