@@ -14,6 +14,7 @@ import (
 	"math/rand"
 	"net/http"
 	"net/url"
+	"sort"
 	"strconv"
 	"strings"
 	"testing"
@@ -205,74 +206,74 @@ func TestMetricsQuery_MinMaxTime(t *testing.T) {
 		{
 			name:         "range query: without range vector, without offset",
 			metricsQuery: rangeRequest,
-			expectedMinT: startTime.UnixMilli() - lookbackDurationMS,
+			expectedMinT: startTime.UnixMilli() - lookbackDurationMS + 1,
 			expectedMaxT: endTime.UnixMilli(),
 		},
 		{
 			name:         "instant query: without range vector, without offset",
 			metricsQuery: instantRequest,
-			expectedMinT: endTime.UnixMilli() - lookbackDurationMS,
+			expectedMinT: endTime.UnixMilli() - lookbackDurationMS + 1,
 			expectedMaxT: endTime.UnixMilli(),
 		},
 		{
 			name:         "range query: with range vector, without offset",
 			metricsQuery: withQuery(t, rangeRequest, fmt.Sprintf("rate(go_goroutines{}[%s])", rangeVectorDurationStr)),
-			expectedMinT: startTime.UnixMilli() - rangeVectorDurationMS, // lookback duration not used with range vectors
+			expectedMinT: startTime.UnixMilli() - rangeVectorDurationMS + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.UnixMilli(),
 		},
 		{
 			name:         "instant query: with range vector, without offset",
 			metricsQuery: withQuery(t, instantRequest, fmt.Sprintf("rate(go_goroutines{}[%s])", rangeVectorDurationStr)),
-			expectedMinT: endTime.UnixMilli() - rangeVectorDurationMS, // lookback duration not used with range vectors
+			expectedMinT: endTime.UnixMilli() - rangeVectorDurationMS + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.UnixMilli(),
 		},
 		{
 			name:         "range query: without range vector, with offset",
 			metricsQuery: withQuery(t, rangeRequest, fmt.Sprintf("go_goroutines{} offset %s", offsetDurationStr)),
-			expectedMinT: startTime.UnixMilli() - offsetDurationMS - lookbackDurationMS,
+			expectedMinT: startTime.UnixMilli() - offsetDurationMS - lookbackDurationMS + 1,
 			expectedMaxT: endTime.UnixMilli() - offsetDurationMS,
 		},
 		{
 			name:         "instant query: without range vector, with offset",
 			metricsQuery: withQuery(t, instantRequest, fmt.Sprintf("go_goroutines{} offset %s", offsetDurationStr)),
-			expectedMinT: endTime.UnixMilli() - offsetDurationMS - lookbackDurationMS,
+			expectedMinT: endTime.UnixMilli() - offsetDurationMS - lookbackDurationMS + 1,
 			expectedMaxT: endTime.UnixMilli() - offsetDurationMS,
 		},
 		{
 			name:         "range query: with range vector, with offset",
 			metricsQuery: withQuery(t, rangeRequest, fmt.Sprintf("rate(go_goroutines{}[%s] offset %s)", rangeVectorDurationStr, offsetDurationStr)),
-			expectedMinT: startTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS, // lookback duration not used with range vectors
+			expectedMinT: startTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.UnixMilli() - offsetDurationMS,
 		},
 		{
 			name:         "instant query: with range vector, with offset",
 			metricsQuery: withQuery(t, instantRequest, fmt.Sprintf("rate(go_goroutines{}[%s] offset %s)", rangeVectorDurationStr, offsetDurationStr)),
-			expectedMinT: endTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS, // lookback duration not used with range vectors
+			expectedMinT: endTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.UnixMilli() - offsetDurationMS,
 		},
 		// permutations with and without range vectors and @ modifiers
 		{
 			name:         "range query: with @ modifer",
 			metricsQuery: withQuery(t, rangeRequest, fmt.Sprintf("go_goroutines{} @ %d", endTime.Add(-atModifierDuration).Unix())),
-			expectedMinT: endTime.Add(-atModifierDuration).UnixMilli() - lookbackDurationMS,
+			expectedMinT: endTime.Add(-atModifierDuration).UnixMilli() - lookbackDurationMS + 1,
 			expectedMaxT: endTime.Add(-atModifierDuration).UnixMilli(),
 		},
 		{
 			name:         "instant query: with @ modifer",
 			metricsQuery: withQuery(t, instantRequest, fmt.Sprintf("go_goroutines{} @ %d", endTime.Add(-atModifierDuration).Unix())),
-			expectedMinT: endTime.Add(-atModifierDuration).UnixMilli() - lookbackDurationMS, // lookback duration not used with range vectors
+			expectedMinT: endTime.Add(-atModifierDuration).UnixMilli() - lookbackDurationMS + 1,
 			expectedMaxT: endTime.Add(-atModifierDuration).UnixMilli(),
 		},
 		{
 			name:         "range query: with range vector, with @ modifer",
 			metricsQuery: withQuery(t, rangeRequest, fmt.Sprintf("go_goroutines{}[%s] @ %d", rangeVectorDurationStr, endTime.Add(-atModifierDuration).Unix())),
-			expectedMinT: endTime.Add(-(atModifierDuration + rangeVectorDuration)).UnixMilli(), // lookback duration not used with range vectors
+			expectedMinT: endTime.Add(-(atModifierDuration + rangeVectorDuration)).UnixMilli() + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.Add(-atModifierDuration).UnixMilli(),
 		},
 		{
 			name:         "instant query: with range vector, with @ modifer",
 			metricsQuery: withQuery(t, instantRequest, fmt.Sprintf("go_goroutines{}[%s] @ %d", rangeVectorDurationStr, endTime.Add(-atModifierDuration).Unix())),
-			expectedMinT: endTime.Add(-(atModifierDuration + rangeVectorDuration)).UnixMilli(), // lookback duration not used with range vectors
+			expectedMinT: endTime.Add(-(atModifierDuration + rangeVectorDuration)).UnixMilli() + 1, // lookback duration not used with range vectors
 			expectedMaxT: endTime.Add(-atModifierDuration).UnixMilli(),
 		},
 	} {
@@ -338,7 +339,7 @@ func TestMetricsQuery_WithStartEnd_TransformConsistency(t *testing.T) {
 			updatedStartTime:    updatedStartTime,
 			updatedEndTime:      updatedEndTime,
 
-			expectedUpdatedMinT: updatedStartTime.UnixMilli(),
+			expectedUpdatedMinT: updatedStartTime.UnixMilli() + 1, // query range is left-open, but minT is inclusive
 			expectedUpdatedMaxT: updatedEndTime.UnixMilli(),
 		},
 		{
@@ -347,7 +348,7 @@ func TestMetricsQuery_WithStartEnd_TransformConsistency(t *testing.T) {
 			updatedStartTime:    updatedEndTime,
 			updatedEndTime:      updatedEndTime,
 
-			expectedUpdatedMinT: updatedEndTime.UnixMilli(),
+			expectedUpdatedMinT: updatedEndTime.UnixMilli() + 1, // query range is left-open, but minT is inclusive
 			expectedUpdatedMaxT: updatedEndTime.UnixMilli(),
 		},
 	} {
@@ -418,7 +419,7 @@ func TestMetricsQuery_WithQuery_WithExpr_TransformConsistency(t *testing.T) {
 			initialMetricsQuery: rangeRequest,
 			updatedQuery:        fmt.Sprintf("rate(go_goroutines{}[%s] offset %s)", rangeVectorDurationStr, offsetDurationStr),
 
-			expectedUpdatedMinT: startTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS,
+			expectedUpdatedMinT: startTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS + 1,
 			expectedUpdatedMaxT: endTime.UnixMilli() - offsetDurationMS,
 			expectedErr:         nil,
 		},
@@ -427,7 +428,7 @@ func TestMetricsQuery_WithQuery_WithExpr_TransformConsistency(t *testing.T) {
 			initialMetricsQuery: instantRequest,
 			updatedQuery:        fmt.Sprintf("rate(go_goroutines{}[%s] offset %s)", rangeVectorDurationStr, offsetDurationStr),
 
-			expectedUpdatedMinT: endTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS,
+			expectedUpdatedMinT: endTime.UnixMilli() - rangeVectorDurationMS - offsetDurationMS + 1,
 			expectedUpdatedMaxT: endTime.UnixMilli() - offsetDurationMS,
 			expectedErr:         nil,
 		},
@@ -474,13 +475,13 @@ func TestMetricsQuery_WithQuery_WithExpr_TransformConsistency(t *testing.T) {
 	}
 }
 
-func TestPrometheusCodec_EncodeLabelsQueryRequest(t *testing.T) {
-	codec := newTestPrometheusCodec()
-
+func TestPrometheusCodec_DecodeEncodeLabelsQueryRequest(t *testing.T) {
 	for _, testCase := range []struct {
 		name                      string
+		propagateHeaders          []string
 		url                       string
-		expectedStruct            LabelsQueryRequest
+		headers                   http.Header
+		expectedStruct            LabelsSeriesQueryRequest
 		expectedGetLabelName      string
 		expectedGetStartOrDefault int64
 		expectedGetEndOrDefault   int64
@@ -648,42 +649,81 @@ func TestPrometheusCodec_EncodeLabelsQueryRequest(t *testing.T) {
 			url:         "/api/v1/label/job/values?limit=-1",
 			expectedErr: "limit parameter must be a positive number: -1",
 		},
+		{
+			name: "propagates headers",
+			headers: http.Header{
+				"X-Special-Header": []string{"some-value"},
+			},
+			url:              "/api/v1/labels?end=1708588800&start=1708502400",
+			propagateHeaders: []string{"X-Special-Header"},
+			expectedStruct: &PrometheusLabelNamesQueryRequest{
+				Path:  "/api/v1/labels",
+				Start: 1708502400 * 1e3,
+				End:   1708588800 * 1e3,
+				Headers: httpHeadersToProm(
+					http.Header{"X-Special-Header": []string{"some-value"}},
+				),
+				LabelMatcherSets: nil,
+			},
+			expectedGetLabelName:      "",
+			expectedGetStartOrDefault: 1708502400 * 1e3,
+			expectedGetEndOrDefault:   1708588800 * 1e3,
+		},
 	} {
 		t.Run(testCase.name, func(t *testing.T) {
 			for _, reqMethod := range []string{http.MethodGet, http.MethodPost} {
+				t.Run(reqMethod, func(t *testing.T) {
+					var r *http.Request
+					var err error
 
-				var r *http.Request
-				var err error
+					expectedStruct := testCase.expectedStruct
 
-				switch reqMethod {
-				case http.MethodGet:
-					r, err = http.NewRequest(reqMethod, testCase.url, nil)
+					switch reqMethod {
+					case http.MethodGet:
+						r, err = http.NewRequest(reqMethod, testCase.url, nil)
+						require.NoError(t, err)
+					case http.MethodPost:
+						parsedURL, _ := url.Parse(testCase.url)
+						r, err = http.NewRequest(reqMethod, parsedURL.Path, strings.NewReader(parsedURL.RawQuery))
+						require.NoError(t, err)
+						r.Header.Set("Content-Type", "application/x-www-form-urlencoded")
+
+						if expectedStruct != nil {
+							headers := append(expectedStruct.GetHeaders(), &PrometheusHeader{"Content-Type", []string{"application/x-www-form-urlencoded"}})
+
+							// Decoding headers also sorts them. We sort here to be able to make assertions on the slice of headers.
+							sort.Slice(headers, func(i, j int) bool { return headers[i].Name < headers[j].Name })
+							expectedStruct, err = expectedStruct.WithHeaders(headers)
+							require.NoError(t, err)
+						}
+					default:
+						t.Fatalf("unsupported HTTP method %q", reqMethod)
+					}
+
+					ctx := user.InjectOrgID(context.Background(), "1")
+					r = r.WithContext(ctx)
+					for k, v := range testCase.headers {
+						for _, v := range v {
+							r.Header.Add(k, v)
+						}
+					}
+
+					codec := newTestPrometheusCodecWithHeaders(testCase.propagateHeaders)
+					reqDecoded, err := codec.DecodeLabelsSeriesQueryRequest(ctx, r)
+					if err != nil || testCase.expectedErr != "" {
+						require.EqualError(t, err, testCase.expectedErr)
+						return
+					}
+
+					require.EqualValues(t, expectedStruct, reqDecoded)
+					require.EqualValues(t, testCase.expectedGetStartOrDefault, reqDecoded.GetStartOrDefault())
+					require.EqualValues(t, testCase.expectedGetEndOrDefault, reqDecoded.GetEndOrDefault())
+					require.EqualValues(t, testCase.expectedLimit, reqDecoded.GetLimit())
+
+					reqEncoded, err := codec.EncodeLabelsSeriesQueryRequest(context.Background(), reqDecoded)
 					require.NoError(t, err)
-				case http.MethodPost:
-					parsedURL, _ := url.Parse(testCase.url)
-					r, err = http.NewRequest(reqMethod, parsedURL.Path, strings.NewReader(parsedURL.RawQuery))
-					require.NoError(t, err)
-					r.Header.Set("Content-Type", "application/x-www-form-urlencoded")
-				default:
-					t.Fatalf("unsupported HTTP method %q", reqMethod)
-				}
-
-				ctx := user.InjectOrgID(context.Background(), "1")
-				r = r.WithContext(ctx)
-
-				reqDecoded, err := codec.DecodeLabelsQueryRequest(ctx, r)
-				if err != nil || testCase.expectedErr != "" {
-					require.EqualError(t, err, testCase.expectedErr)
-					return
-				}
-				require.EqualValues(t, testCase.expectedStruct, reqDecoded)
-				require.EqualValues(t, testCase.expectedGetStartOrDefault, reqDecoded.GetStartOrDefault())
-				require.EqualValues(t, testCase.expectedGetEndOrDefault, reqDecoded.GetEndOrDefault())
-				require.EqualValues(t, testCase.expectedLimit, reqDecoded.GetLimit())
-
-				reqEncoded, err := codec.EncodeLabelsQueryRequest(context.Background(), reqDecoded)
-				require.NoError(t, err)
-				require.EqualValues(t, testCase.url, reqEncoded.RequestURI)
+					require.EqualValues(t, testCase.url, reqEncoded.RequestURI)
+				})
 			}
 		})
 	}
@@ -692,7 +732,7 @@ func TestPrometheusCodec_EncodeLabelsQueryRequest(t *testing.T) {
 func TestPrometheusCodec_EncodeMetricsQueryRequest_AcceptHeader(t *testing.T) {
 	for _, queryResultPayloadFormat := range allFormats {
 		t.Run(queryResultPayloadFormat, func(t *testing.T) {
-			codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, queryResultPayloadFormat)
+			codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, queryResultPayloadFormat, nil)
 			req := PrometheusInstantQueryRequest{}
 			encodedRequest, err := codec.EncodeMetricsQueryRequest(context.Background(), &req)
 			require.NoError(t, err)
@@ -712,7 +752,7 @@ func TestPrometheusCodec_EncodeMetricsQueryRequest_AcceptHeader(t *testing.T) {
 func TestPrometheusCodec_EncodeMetricsQueryRequest_ReadConsistency(t *testing.T) {
 	for _, consistencyLevel := range api.ReadConsistencies {
 		t.Run(consistencyLevel, func(t *testing.T) {
-			codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatProtobuf)
+			codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatProtobuf, nil)
 			ctx := api.ContextWithReadConsistencyLevel(context.Background(), consistencyLevel)
 			encodedRequest, err := codec.EncodeMetricsQueryRequest(ctx, &PrometheusInstantQueryRequest{})
 			require.NoError(t, err)
@@ -724,7 +764,7 @@ func TestPrometheusCodec_EncodeMetricsQueryRequest_ReadConsistency(t *testing.T)
 func TestPrometheusCodec_EncodeMetricsQueryRequest_ShouldPropagateHeadersInAllowList(t *testing.T) {
 	const notAllowedHeader = "X-Some-Name"
 
-	codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatProtobuf)
+	codec := NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatProtobuf, nil)
 	expectedOffsets := map[int32]int64{0: 1, 1: 2}
 
 	req, err := codec.EncodeMetricsQueryRequest(context.Background(), &PrometheusInstantQueryRequest{
@@ -761,10 +801,10 @@ func TestPrometheusCodec_EncodeResponse_ContentNegotiation(t *testing.T) {
 		Error:     "something went wrong",
 	}
 
-	jsonBody, err := jsonFormatter{}.EncodeResponse(testResponse)
+	jsonBody, err := jsonFormatter{}.EncodeQueryResponse(testResponse)
 	require.NoError(t, err)
 
-	protobufBody, err := protobufFormatter{}.EncodeResponse(testResponse)
+	protobufBody, err := protobufFormatter{}.EncodeQueryResponse(testResponse)
 	require.NoError(t, err)
 
 	scenarios := map[string]struct {
@@ -816,7 +856,7 @@ func TestPrometheusCodec_EncodeResponse_ContentNegotiation(t *testing.T) {
 			require.NoError(t, err)
 			req.Header.Set("Accept", scenario.acceptHeader)
 
-			encodedResponse, err := codec.EncodeResponse(context.Background(), req, testResponse)
+			encodedResponse, err := codec.EncodeMetricsQueryResponse(context.Background(), req, testResponse)
 			require.Equal(t, scenario.expectedError, err)
 
 			if scenario.expectedError == nil {
@@ -919,7 +959,7 @@ func TestPrometheusCodec_DecodeResponse_Errors(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			codec := newTestPrometheusCodec()
 
-			_, err := codec.DecodeResponse(context.Background(), testCase.response, nil, testutil.NewTestingLogger(t))
+			_, err := codec.DecodeMetricsQueryResponse(context.Background(), testCase.response, nil, testutil.NewTestingLogger(t))
 			require.Error(t, err)
 			require.True(t, apierror.IsAPIError(err))
 			resp, ok := apierror.HTTPResponseFromError(err)
@@ -948,7 +988,7 @@ func TestPrometheusCodec_DecodeResponse_ContentTypeHandling(t *testing.T) {
 	} {
 		t.Run(tc.name, func(t *testing.T) {
 			reg := prometheus.NewPedanticRegistry()
-			codec := NewPrometheusCodec(reg, 0*time.Minute, formatJSON)
+			codec := NewPrometheusCodec(reg, 0*time.Minute, formatJSON, nil)
 
 			resp := prometheusAPIResponse{}
 			body, err := json.Marshal(resp)
@@ -960,7 +1000,7 @@ func TestPrometheusCodec_DecodeResponse_ContentTypeHandling(t *testing.T) {
 				ContentLength: int64(len(body)),
 			}
 
-			_, err = codec.DecodeResponse(context.Background(), httpResponse, nil, log.NewNopLogger())
+			_, err = codec.DecodeMetricsQueryResponse(context.Background(), httpResponse, nil, log.NewNopLogger())
 			require.Equal(t, tc.expectedErr, err)
 		})
 	}
@@ -1441,7 +1481,7 @@ func BenchmarkPrometheusCodec_DecodeResponse(b *testing.B) {
 	b.ReportAllocs()
 
 	for n := 0; n < b.N; n++ {
-		_, err := codec.DecodeResponse(context.Background(), &http.Response{
+		_, err := codec.DecodeMetricsQueryResponse(context.Background(), &http.Response{
 			StatusCode:    200,
 			Body:          io.NopCloser(bytes.NewReader(encodedRes)),
 			ContentLength: int64(len(encodedRes)),
@@ -1470,7 +1510,7 @@ func BenchmarkPrometheusCodec_EncodeResponse(b *testing.B) {
 	b.ReportAllocs()
 
 	for n := 0; n < b.N; n++ {
-		_, err := codec.EncodeResponse(context.Background(), req, res)
+		_, err := codec.EncodeMetricsQueryResponse(context.Background(), req, res)
 		require.NoError(b, err)
 	}
 }
@@ -1618,9 +1658,9 @@ func Test_DecodeOptions(t *testing.T) {
 	}
 }
 
-// TestPrometheusCodec_DecodeEncode tests that decoding and re-encoding a
-// request does not lose relevant information about the original request.
-func TestPrometheusCodec_DecodeEncode(t *testing.T) {
+// TestPrometheusCodec_DecodeEncode_Metrics tests that decoding and re-encoding a
+// metrics query request does not lose relevant information about the original request.
+func TestPrometheusCodec_DecodeEncode_Metrics(t *testing.T) {
 	codec := newTestPrometheusCodec().(prometheusCodec)
 	for _, tt := range []struct {
 		name    string
@@ -1678,6 +1718,133 @@ func TestPrometheusCodec_DecodeEncode(t *testing.T) {
 	}
 }
 
+// TestPrometheusCodec_DecodeEncodeMultipleTimes_Labels tests that decoding and re-encoding a
+// labels query request multiple times does not lose relevant information about the original request.
+func TestPrometheusCodec_DecodeEncodeMultipleTimes_Labels(t *testing.T) {
+
+	defaultHeaders := httpHeadersToProm(http.Header{
+		"Accept": {"application/json"},
+	})
+	codec := newTestPrometheusCodec().(prometheusCodec)
+	for _, tc := range []struct {
+		name     string
+		queryURL string
+		request  LabelsSeriesQueryRequest
+	}{
+		{
+			name:     "label names - minimal",
+			queryURL: "/api/v1/labels?end=1708588800&start=1708502400",
+			request: &PrometheusLabelNamesQueryRequest{
+				Path:    "/api/v1/labels",
+				Headers: defaultHeaders,
+				Start:   1708502400000,
+				End:     1708588800000,
+			},
+		},
+		{
+			name:     "label names - all",
+			queryURL: "/api/v1/labels?end=1708588800&limit=10&match%5B%5D=go_goroutines%7Bcontainer%3D~%22quer.%2A%22%7D&match%5B%5D=go_goroutines%7Bcontainer%21%3D%22query-scheduler%22%7D&start=1708502400",
+			request: &PrometheusLabelNamesQueryRequest{
+				Path:    "/api/v1/labels",
+				Headers: defaultHeaders,
+				Start:   1708502400000,
+				End:     1708588800000,
+				LabelMatcherSets: []string{
+					"go_goroutines{container=~\"quer.*\"}",
+					"go_goroutines{container!=\"query-scheduler\"}",
+				},
+				Limit: 10,
+			},
+		},
+		{
+			name:     "label values - minimal",
+			queryURL: "/api/v1/label/job/values?end=1708588800&start=1708502400",
+			request: &PrometheusLabelValuesQueryRequest{
+				Path:      "/api/v1/label/job/values",
+				Headers:   defaultHeaders,
+				LabelName: "job",
+				Start:     1708502400000,
+				End:       1708588800000,
+			},
+		},
+		{
+			name:     "label values - all",
+			queryURL: "/api/v1/label/job/values?end=1708588800&limit=10&match%5B%5D=go_goroutines%7Bcontainer%3D~%22quer.%2A%22%7D&match%5B%5D=go_goroutines%7Bcontainer%21%3D%22query-scheduler%22%7D&start=1708502400",
+			request: &PrometheusLabelValuesQueryRequest{
+				Path:      "/api/v1/label/job/values",
+				Headers:   defaultHeaders,
+				LabelName: "job",
+				Start:     1708502400000,
+				End:       1708588800000,
+				LabelMatcherSets: []string{
+					"go_goroutines{container=~\"quer.*\"}",
+					"go_goroutines{container!=\"query-scheduler\"}",
+				},
+				Limit: 10,
+			},
+		},
+		{
+			name:     "series - minimal",
+			queryURL: "/api/v1/series?end=1708588800&match%5B%5D=go_goroutines%7Bcontainer%21%3D%22query-scheduler%22%7D&start=1708502400",
+			request: &PrometheusSeriesQueryRequest{
+				Path:    "/api/v1/series",
+				Headers: defaultHeaders,
+				Start:   1708502400000,
+				End:     1708588800000,
+				LabelMatcherSets: []string{
+					"go_goroutines{container!=\"query-scheduler\"}",
+				},
+			},
+		},
+		{
+			name:     "series - all",
+			queryURL: "/api/v1/series?end=1708588800&limit=10&match%5B%5D=go_goroutines%7Bcontainer%3D~%22quer.%2A%22%7D&match%5B%5D=go_goroutines%7Bcontainer%21%3D%22query-scheduler%22%7D&start=1708502400",
+			request: &PrometheusSeriesQueryRequest{
+				Path:    "/api/v1/series",
+				Headers: defaultHeaders,
+				Start:   1708502400000,
+				End:     1708588800000,
+				LabelMatcherSets: []string{
+					"go_goroutines{container=~\"quer.*\"}",
+					"go_goroutines{container!=\"query-scheduler\"}",
+				},
+				Limit: 10,
+			},
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			t.Parallel()
+
+			expected, err := http.NewRequest("GET", tc.queryURL, nil)
+			require.NoError(t, err)
+			expected.Body = http.NoBody
+			expected.Header = make(http.Header)
+			// This header is set by EncodeLabelsSeriesQueryRequest according to the codec's config, so we
+			// should always expect it to be present on the re-encoded request.
+			expected.Header.Set("Accept", "application/json")
+			ctx := context.Background()
+
+			decoded, err := codec.DecodeLabelsSeriesQueryRequest(ctx, expected)
+			require.NoError(t, err)
+			assert.Equal(t, tc.request, decoded)
+
+			encoded, err := codec.EncodeLabelsSeriesQueryRequest(ctx, decoded)
+			require.NoError(t, err)
+			assert.Equal(t, expected.URL, encoded.URL)
+			assert.Equal(t, expected.Header, encoded.Header)
+
+			decoded, err = codec.DecodeLabelsSeriesQueryRequest(ctx, encoded)
+			require.NoError(t, err)
+			assert.Equal(t, tc.request, decoded)
+
+			encoded, err = codec.EncodeLabelsSeriesQueryRequest(ctx, decoded)
+			require.NoError(t, err)
+			assert.Equal(t, expected.URL, encoded.URL)
+			assert.Equal(t, expected.Header, encoded.Header)
+		})
+	}
+}
+
 func TestPrometheusCodec_DecodeMultipleTimes(t *testing.T) {
 	const query = "sum by (namespace) (container_memory_rss)"
 	t.Run("instant query", func(t *testing.T) {
@@ -1731,7 +1898,11 @@ func TestPrometheusCodec_DecodeMultipleTimes(t *testing.T) {
 }
 
 func newTestPrometheusCodec() Codec {
-	return NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatJSON)
+	return newTestPrometheusCodecWithHeaders(nil)
+}
+
+func newTestPrometheusCodecWithHeaders(propagateHeaders []string) Codec {
+	return NewPrometheusCodec(prometheus.NewPedanticRegistry(), 0*time.Minute, formatJSON, propagateHeaders)
 }
 
 func mustSucceed[T any](value T, err error) T {
