@@ -30,6 +30,7 @@ import (
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/testutil"
 	"github.com/prometheus/common/model"
+	"github.com/prometheus/common/promslog"
 	"github.com/prometheus/prometheus/prompb"
 	"github.com/prometheus/prometheus/promql"
 	"github.com/stretchr/testify/assert"
@@ -82,7 +83,7 @@ func TestTripperware_RangeQuery(t *testing.T) {
 		newTestPrometheusCodec(),
 		nil,
 		promql.EngineOpts{
-			Logger:     log.NewNopLogger(),
+			Logger:     promslog.NewNopLogger(),
 			Reg:        nil,
 			MaxSamples: 1000,
 			Timeout:    time.Minute,
@@ -135,7 +136,7 @@ func TestTripperware_InstantQuery(t *testing.T) {
 		codec,
 		nil,
 		promql.EngineOpts{
-			Logger:     log.NewNopLogger(),
+			Logger:     promslog.NewNopLogger(),
 			Reg:        nil,
 			MaxSamples: 1000,
 			Timeout:    time.Minute,
@@ -154,7 +155,7 @@ func TestTripperware_InstantQuery(t *testing.T) {
 			return nil, err
 		}
 
-		return codec.EncodeResponse(r.Context(), r, &PrometheusResponse{
+		return codec.EncodeMetricsQueryResponse(r.Context(), r, &PrometheusResponse{
 			Status: "success",
 			Data: &PrometheusData{
 				ResultType: "vector",
@@ -185,26 +186,6 @@ func TestTripperware_InstantQuery(t *testing.T) {
 		}, res)
 	})
 
-	t.Run("specific time param with form being already parsed", func(t *testing.T) {
-		ts := time.Date(2021, 1, 2, 3, 4, 5, 0, time.UTC)
-
-		formParserRoundTripper := RoundTripFunc(func(r *http.Request) (*http.Response, error) {
-			assert.NoError(t, r.ParseForm())
-			return tripper.RoundTrip(r)
-		})
-		queryClient, err := api.NewClient(api.Config{Address: "http://localhost", RoundTripper: formParserRoundTripper})
-		require.NoError(t, err)
-		api := v1.NewAPI(queryClient)
-
-		res, _, err := api.Query(ctx, `sum(increase(we_dont_care_about_this[1h])) by (foo)`, ts)
-		require.NoError(t, err)
-		require.IsType(t, model.Vector{}, res)
-		require.NotEmpty(t, res.(model.Vector))
-
-		resultTime := res.(model.Vector)[0].Timestamp.Time()
-		require.Equal(t, ts.Unix(), resultTime.Unix())
-	})
-
 	t.Run("default time param happy case", func(t *testing.T) {
 		queryClient, err := api.NewClient(api.Config{Address: "http://localhost", RoundTripper: tripper})
 		require.NoError(t, err)
@@ -219,24 +200,6 @@ func TestTripperware_InstantQuery(t *testing.T) {
 		require.InDelta(t, time.Now().Unix(), resultTime.Unix(), 1)
 	})
 
-	t.Run("default time param with form being already parsed", func(t *testing.T) {
-		formParserRoundTripper := RoundTripFunc(func(r *http.Request) (*http.Response, error) {
-			assert.NoError(t, r.ParseForm())
-			return tripper.RoundTrip(r)
-		})
-		queryClient, err := api.NewClient(api.Config{Address: "http://localhost", RoundTripper: formParserRoundTripper})
-		require.NoError(t, err)
-		api := v1.NewAPI(queryClient)
-
-		res, _, err := api.Query(ctx, `sum(increase(we_dont_care_about_this[1h])) by (foo)`, time.Time{})
-		require.NoError(t, err)
-		require.IsType(t, model.Vector{}, res)
-		require.NotEmpty(t, res.(model.Vector))
-
-		resultTime := res.(model.Vector)[0].Timestamp.Time()
-		require.InDelta(t, time.Now().Unix(), resultTime.Unix(), 1)
-	})
-
 	t.Run("post form time param takes precedence over query time param ", func(t *testing.T) {
 		postFormTimeParam := time.Date(2021, 1, 2, 3, 4, 5, 0, time.UTC)
 
@@ -505,7 +468,7 @@ func TestTripperware_Metrics(t *testing.T) {
 				newTestPrometheusCodec(),
 				nil,
 				promql.EngineOpts{
-					Logger:     log.NewNopLogger(),
+					Logger:     promslog.NewNopLogger(),
 					Reg:        nil,
 					MaxSamples: 1000,
 					Timeout:    time.Minute,
@@ -541,11 +504,13 @@ func TestMiddlewaresConsistency(t *testing.T) {
 	cfg.CacheResults = true
 	cfg.ShardedQueries = true
 	cfg.PrunedQueries = true
+	cfg.BlockPromQLExperimentalFunctions = true
 
 	// Ensure all features are enabled, so that we assert on all middlewares.
 	require.NotZero(t, cfg.CacheResults)
 	require.NotZero(t, cfg.ShardedQueries)
 	require.NotZero(t, cfg.PrunedQueries)
+	require.NotZero(t, cfg.BlockPromQLExperimentalFunctions)
 	require.NotZero(t, cfg.SplitQueriesByInterval)
 	require.NotZero(t, cfg.MaxRetries)
 
@@ -585,6 +550,7 @@ func TestMiddlewaresConsistency(t *testing.T) {
 				"splitInstantQueryByIntervalMiddleware", // Not applicable because specific to instant queries.
 				"stepAlignMiddleware",                   // Not applicable because remote read requests don't take step in account when running in Mimir.
 				"pruneMiddleware",                       // No query pruning support.
+				"experimentalFunctionsMiddleware",       // No blocking for PromQL experimental functions as it is executed remotely.
 			},
 		},
 	}
@@ -767,7 +733,7 @@ func TestTripperware_RemoteRead(t *testing.T) {
 				newTestPrometheusCodec(),
 				nil,
 				promql.EngineOpts{
-					Logger:     log.NewNopLogger(),
+					Logger:     promslog.NewNopLogger(),
 					Reg:        nil,
 					MaxSamples: 1000,
 					Timeout:    time.Minute,
@@ -830,7 +796,7 @@ func TestTripperware_ShouldSupportReadConsistencyOffsetsInjection(t *testing.T)
 		},
 		"cardinality label values": {
 			makeRequest: func() *http.Request {
-				return httptest.NewRequest("GET", cardinalityLabelValuesPathSuffix, nil)
+				return httptest.NewRequest("GET", cardinalityLabelValuesPathSuffix+"?label_names[]=foo", nil)
 			},
 		},
 		"cardinality active series": {
@@ -899,10 +865,10 @@ func TestTripperware_ShouldSupportReadConsistencyOffsetsInjection(t *testing.T)
 		}),
 		log.NewNopLogger(),
 		mockLimits{},
-		NewPrometheusCodec(nil, 0, formatJSON),
+		NewPrometheusCodec(nil, 0, formatJSON, nil),
 		nil,
 		promql.EngineOpts{
-			Logger:     log.NewNopLogger(),
+			Logger:     promslog.NewNopLogger(),
 			Reg:        nil,
 			MaxSamples: 1000,
 			Timeout:    time.Minute,
