@@ -377,8 +377,8 @@ func TestInstantQuerySplittingCorrectness(t *testing.T) {
 					expectedSplitQueries:         0,
 					expectedSkippedNonSplittable: 1,
 				},
-				"holt_winters": {
-					query:                        `holt_winters(metric_counter[1m], 0.5, 0.9)`,
+				"double_exponential_smoothing": {
+					query:                        `double_exponential_smoothing(metric_counter[1m], 0.5, 0.9)`,
 					expectedSplitQueries:         0,
 					expectedSkippedNonSplittable: 1,
 				},
@@ -516,8 +516,9 @@ func TestInstantQuerySplittingCorrectness(t *testing.T) {
 							reg := prometheus.NewPedanticRegistry()
 							engine := newEngine()
 							downstream := &downstreamHandler{
-								engine:    engine,
-								queryable: queryable,
+								engine:                                  engine,
+								queryable:                               queryable,
+								includePositionInformationInAnnotations: true,
 							}
 
 							// Run the query with the normal engine
@@ -531,6 +532,12 @@ func TestInstantQuerySplittingCorrectness(t *testing.T) {
 							require.NotEmpty(t, expectedPrometheusRes.Data.Result)
 							requireValidSamples(t, expectedPrometheusRes.Data.Result)
 
+							if testData.expectedSplitQueries > 0 {
+								// Remove position information from annotations, to mirror what we expect from the split queries below.
+								removeAllAnnotationPositionInformation(expectedPrometheusRes.Infos)
+								removeAllAnnotationPositionInformation(expectedPrometheusRes.Warnings)
+							}
+
 							splittingware := newSplitInstantQueryByIntervalMiddleware(mockLimits{splitInstantQueriesInterval: 1 * time.Minute}, log.NewNopLogger(), engine, reg)
 
 							// Run the query with splitting
