@@ -241,8 +241,8 @@ func TestDistributorQuerier_Select_MixedChunkseriesTimeseriesAndStreamingResults
 	}
 
 	streamReader := createTestStreamReader([]client.QueryStreamSeriesChunks{
-		{SeriesIndex: 0, Chunks: convertToChunks(t, samplesToInterface(s4))},
-		{SeriesIndex: 1, Chunks: convertToChunks(t, samplesToInterface(s3))},
+		{SeriesIndex: 0, Chunks: convertToChunks(t, samplesToInterface(s4), false)},
+		{SeriesIndex: 1, Chunks: convertToChunks(t, samplesToInterface(s3), false)},
 	})
 
 	d := &mockDistributor{}
@@ -251,11 +251,11 @@ func TestDistributorQuerier_Select_MixedChunkseriesTimeseriesAndStreamingResults
 			Chunkseries: []client.TimeSeriesChunk{
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "one"}},
-					Chunks: convertToChunks(t, samplesToInterface(s1)),
+					Chunks: convertToChunks(t, samplesToInterface(s1), false),
 				},
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "two"}},
-					Chunks: convertToChunks(t, samplesToInterface(s1)),
+					Chunks: convertToChunks(t, samplesToInterface(s1), false),
 				},
 			},
 
@@ -319,6 +319,18 @@ func genTestFloatHistogram(timestamp int64, value int) mimirpb.Histogram {
 	return mimirpb.FromFloatHistogramToHistogramProto(timestamp, test.GenerateTestFloatHistogram(value))
 }
 
+func genExpectedHistogram(timestamp int64, value int, resetHint mimirpb.Histogram_ResetHint) mimirpb.Histogram {
+	h := mimirpb.FromHistogramToHistogramProto(timestamp, test.GenerateTestHistogram(value))
+	h.ResetHint = resetHint
+	return h
+}
+
+func genExpectedFloatHistogram(timestamp int64, value int, resetHint mimirpb.Histogram_ResetHint) mimirpb.Histogram {
+	fh := mimirpb.FromFloatHistogramToHistogramProto(timestamp, test.GenerateTestFloatHistogram(value))
+	fh.ResetHint = resetHint
+	return fh
+}
+
 func TestDistributorQuerier_Select_MixedFloatAndIntegerHistograms(t *testing.T) {
 	const (
 		mint = 0
@@ -332,6 +344,16 @@ func TestDistributorQuerier_Select_MixedFloatAndIntegerHistograms(t *testing.T)
 		genTestFloatHistogram(4000, 4),
 		genTestHistogram(5000, 5),
 	}
+	// When s1 is encoded to chunks, the counter reset hint is set by the chunk
+	// appender.
+	expectS1 := []mimirpb.Histogram{
+		genExpectedHistogram(1000, 1, mimirpb.Histogram_UNKNOWN),
+		genExpectedHistogram(2000, 2, mimirpb.Histogram_NO),
+		genExpectedHistogram(3000, 3, mimirpb.Histogram_NO),
+		genExpectedFloatHistogram(4000, 4, mimirpb.Histogram_UNKNOWN),
+		genExpectedHistogram(5000, 5, mimirpb.Histogram_UNKNOWN),
+	}
+
 	s2 := []mimirpb.Histogram{
 		genTestHistogram(1000, 1),
 		genTestFloatHistogram(2500, 25),
@@ -355,11 +377,11 @@ func TestDistributorQuerier_Select_MixedFloatAndIntegerHistograms(t *testing.T)
 			Chunkseries: []client.TimeSeriesChunk{
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "one"}},
-					Chunks: convertToChunks(t, histogramsToInterface(s1)),
+					Chunks: convertToChunks(t, histogramsToInterface(s1), false),
 				},
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "two"}},
-					Chunks: convertToChunks(t, histogramsToInterface(s1)),
+					Chunks: convertToChunks(t, histogramsToInterface(s1), false),
 				},
 			},
 
@@ -385,7 +407,7 @@ func TestDistributorQuerier_Select_MixedFloatAndIntegerHistograms(t *testing.T)
 	require.NoError(t, seriesSet.Err())
 
 	require.True(t, seriesSet.Next())
-	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "one"), histogramsToInterface(s1))
+	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "one"), histogramsToInterface(expectS1))
 
 	require.True(t, seriesSet.Next())
 	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "three"), histogramsToInterface(s1))
@@ -414,6 +436,13 @@ func TestDistributorQuerier_Select_MixedHistogramsAndFloatSamples(t *testing.T)
 		genTestFloatHistogram(6000, 60),
 		genTestFloatHistogram(8000, 80),
 	}
+	// When h1 is encoded to chunks, the counter reset hint is set by the chunk
+	// appender.
+	expectH1 := []mimirpb.Histogram{
+		genExpectedFloatHistogram(5500, 55, mimirpb.Histogram_UNKNOWN),
+		genExpectedFloatHistogram(6000, 60, mimirpb.Histogram_NO),
+		genExpectedFloatHistogram(8000, 80, mimirpb.Histogram_NO),
+	}
 	s2 := []mimirpb.Sample{
 		{Value: 1, TimestampMs: 1000},
 		{Value: 2.5, TimestampMs: 2500},
@@ -447,11 +476,11 @@ func TestDistributorQuerier_Select_MixedHistogramsAndFloatSamples(t *testing.T)
 			Chunkseries: []client.TimeSeriesChunk{
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "one"}},
-					Chunks: convertToChunks(t, append(samplesToInterface(s1), histogramsToInterface(h1)...)),
+					Chunks: convertToChunks(t, append(samplesToInterface(s1), histogramsToInterface(h1)...), false),
 				},
 				{
 					Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "two"}},
-					Chunks: convertToChunks(t, append(samplesToInterface(s1), histogramsToInterface(h1)...)),
+					Chunks: convertToChunks(t, append(samplesToInterface(s1), histogramsToInterface(h1)...), false),
 				},
 			},
 
@@ -479,7 +508,7 @@ func TestDistributorQuerier_Select_MixedHistogramsAndFloatSamples(t *testing.T)
 	require.NoError(t, seriesSet.Err())
 
 	require.True(t, seriesSet.Next())
-	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "one"), append(samplesToInterface(s1), histogramsToInterface(h1)...))
+	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "one"), append(samplesToInterface(s1), histogramsToInterface(expectH1)...))
 
 	require.True(t, seriesSet.Next())
 	verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "three"), append(samplesToInterface(s1), histogramsToInterface(h1)...))
@@ -491,6 +520,123 @@ func TestDistributorQuerier_Select_MixedHistogramsAndFloatSamples(t *testing.T)
 	require.NoError(t, seriesSet.Err())
 }
 
+func TestDistributorQuerier_Select_CounterResets(t *testing.T) {
+	makeHistogram := func(ts, val int64, hint mimirpb.Histogram_ResetHint) mimirpb.Histogram {
+		return mimirpb.Histogram{
+			Count: &mimirpb.Histogram_CountInt{CountInt: uint64(val)},
+			Sum:   float64(val),
+			PositiveSpans: []mimirpb.BucketSpan{
+				{Offset: 0, Length: 1},
+			},
+			PositiveDeltas: []int64{val},
+			Timestamp:      ts,
+			ResetHint:      hint,
+		}
+	}
+
+	for _, tc := range []struct {
+		name                 string
+		chunks               []client.Chunk
+		queryStart, queryEnd int64
+		expectedSamples      []mimirpb.Histogram
+	}{
+		{
+			// This might happen is when an in-order chunk and OOO chunk are returned from the same ingester.
+			name: "overlapping chunks",
+			chunks: append(
+				convertToChunks(t, histogramsToInterface([]mimirpb.Histogram{
+					makeHistogram(100, 40, mimirpb.Histogram_UNKNOWN),
+					makeHistogram(700, 50, mimirpb.Histogram_NO),
+				}), false),
+				convertToChunks(t, histogramsToInterface([]mimirpb.Histogram{
+					makeHistogram(200, 20, mimirpb.Histogram_UNKNOWN),
+					makeHistogram(300, 60, mimirpb.Histogram_NO),
+					makeHistogram(400, 70, mimirpb.Histogram_NO),
+					makeHistogram(500, 80, mimirpb.Histogram_NO),
+					makeHistogram(600, 90, mimirpb.Histogram_NO),
+				}), false)...),
+			expectedSamples: []mimirpb.Histogram{
+				makeHistogram(100, 40, mimirpb.Histogram_UNKNOWN),
+				makeHistogram(200, 20, mimirpb.Histogram_UNKNOWN),
+				makeHistogram(300, 60, mimirpb.Histogram_NO),
+				makeHistogram(400, 70, mimirpb.Histogram_NO),
+				makeHistogram(500, 80, mimirpb.Histogram_NO),
+				makeHistogram(600, 90, mimirpb.Histogram_NO),
+				makeHistogram(700, 50, mimirpb.Histogram_UNKNOWN),
+			},
+		},
+		{
+			// This might happen when one of the ingesters hasn't ingested all the samples.
+			name: "duplicate samples in separate chunks",
+			chunks: append(
+				convertToChunks(t, histogramsToInterface([]mimirpb.Histogram{
+					makeHistogram(100, 40, mimirpb.Histogram_UNKNOWN),
+					makeHistogram(300, 20, mimirpb.Histogram_NO),
+				}), true),
+				convertToChunks(t, histogramsToInterface([]mimirpb.Histogram{
+					makeHistogram(100, 40, mimirpb.Histogram_UNKNOWN),
+					makeHistogram(200, 30, mimirpb.Histogram_NO),
+					makeHistogram(300, 20, mimirpb.Histogram_NO),
+				}), true)...),
+			expectedSamples: []mimirpb.Histogram{
+				makeHistogram(100, 40, mimirpb.Histogram_UNKNOWN),
+				makeHistogram(200, 30, mimirpb.Histogram_UNKNOWN),
+				makeHistogram(300, 20, mimirpb.Histogram_UNKNOWN),
+			},
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			responseTypes := map[string]struct {
+				combinedResponse client.CombinedQueryStreamResponse
+			}{
+				"chunkseries": {
+					combinedResponse: client.CombinedQueryStreamResponse{
+						Chunkseries: []client.TimeSeriesChunk{
+							{
+								Labels: []mimirpb.LabelAdapter{{Name: labels.MetricName, Value: "one"}},
+								Chunks: tc.chunks,
+							},
+						},
+					},
+				},
+				"streamingseries": {
+					combinedResponse: client.CombinedQueryStreamResponse{
+						StreamingSeries: []client.StreamingSeries{
+							{
+								Labels: labels.FromStrings(labels.MetricName, "one"),
+								Sources: []client.StreamingSeriesSource{
+									{SeriesIndex: 0, StreamReader: createTestStreamReader([]client.QueryStreamSeriesChunks{
+										{SeriesIndex: 0, Chunks: tc.chunks},
+									})},
+								},
+							},
+						},
+					},
+				},
+			}
+
+			for responseName, responseType := range responseTypes {
+				t.Run(responseName, func(t *testing.T) {
+					d := &mockDistributor{}
+					d.On("QueryStream", mock.Anything, mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return(responseType.combinedResponse, nil)
+
+					ctx := user.InjectOrgID(context.Background(), "0")
+					queryable := NewDistributorQueryable(d, newMockConfigProvider(0), stats.NewQueryMetrics(prometheus.NewPedanticRegistry()), log.NewNopLogger())
+					querier, err := queryable.Querier(tc.queryStart, tc.queryEnd)
+					require.NoError(t, err)
+
+					seriesSet := querier.Select(ctx, true, &storage.SelectHints{Start: tc.queryStart, End: tc.queryEnd}, labels.MustNewMatcher(labels.MatchRegexp, labels.MetricName, ".*"))
+					require.True(t, seriesSet.Next())
+					require.NoError(t, seriesSet.Err())
+
+					verifySeries(t, seriesSet.At(), labels.FromStrings(labels.MetricName, "one"), histogramsToInterface(tc.expectedSamples))
+					require.False(t, seriesSet.Next())
+				})
+			}
+		})
+	}
+}
+
 func TestDistributorQuerier_LabelNames(t *testing.T) {
 	const mint, maxt = 0, 10
 
@@ -623,21 +769,23 @@ func verifySeries(t *testing.T, series storage.Series, l labels.Labels, samples
 	require.Nil(t, it.Err())
 }
 
-func convertToChunks(t *testing.T, samples []interface{}) []client.Chunk {
+func convertToChunks(t *testing.T, samples []interface{}, allowOverflow bool) []client.Chunk {
 	var (
 		overflow chunk.EncodedChunk
+		enc      chunk.Encoding
+		ts       int64
 		err      error
 	)
 
 	chunks := []chunk.Chunk{}
-	ensureChunk := func(enc chunk.Encoding, ts int64) {
-		if len(chunks) == 0 || chunks[len(chunks)-1].Data.Encoding() != enc {
-			c, err := chunk.NewForEncoding(enc)
+	ensureChunk := func(reqEnc chunk.Encoding, reqTs int64) {
+		enc = reqEnc
+		ts = reqTs
+		if len(chunks) == 0 || chunks[len(chunks)-1].Data.Encoding() != reqEnc {
+			c, err := chunk.NewForEncoding(reqEnc)
 			require.NoError(t, err)
-			chunks = append(chunks, chunk.NewChunk(labels.EmptyLabels(), c, model.Time(ts), model.Time(ts)))
-			return
+			chunks = append(chunks, chunk.NewChunk(labels.EmptyLabels(), c, model.Time(reqTs), model.Time(reqTs)))
 		}
-		chunks[len(chunks)-1].Through = model.Time(ts)
 	}
 
 	for _, s := range samples {
@@ -657,7 +805,18 @@ func convertToChunks(t *testing.T, samples []interface{}) []client.Chunk {
 			t.Errorf("convertToChunks - unhandled type: %T", s)
 		}
 		require.NoError(t, err)
-		require.Nil(t, overflow)
+		if overflow == nil {
+			chunks[len(chunks)-1].Through = model.Time(ts)
+			continue
+		}
+		if !allowOverflow {
+			require.Nil(t, overflow)
+			continue
+		}
+		c, err := chunk.NewForEncoding(enc)
+		require.NoError(t, err)
+		chunks = append(chunks, chunk.NewChunk(labels.EmptyLabels(), c, model.Time(ts), model.Time(ts)))
+		chunks[len(chunks)-1].Data = overflow
 	}
 
 	clientChunks, err := client.ToChunks(chunks)
