@@ -11,7 +11,6 @@ import (
 	"io"
 	"net/http"
 	"net/url"
-	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -64,6 +63,7 @@ type Alert struct {
 // RuleDiscovery has info for all rules
 type RuleDiscovery struct {
 	RuleGroups []*RuleGroup `json:"groups"`
+	NextToken  string       `json:"groupNextToken,omitempty"`
 }
 
 // RuleGroup has info for rules which are part of a group
@@ -166,11 +166,29 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 		return
 	}
 
+	excludeAlerts, err := parseExcludeAlerts(req)
+	if err != nil {
+		respondInvalidRequest(logger, w, "invalid exclude_alerts parameter")
+		return
+	}
+
+	var maxGroups int
+	if maxGroupsVal := req.URL.Query().Get("group_limit"); maxGroupsVal != "" {
+		maxGroups, err = strconv.Atoi(maxGroupsVal)
+		if err != nil || maxGroups < 0 {
+			respondInvalidRequest(logger, w, "invalid group limit value")
+			return
+		}
+	}
+
 	rulesReq := RulesRequest{
-		Filter:    AnyRule,
-		RuleName:  req.URL.Query()["rule_name"],
-		RuleGroup: req.URL.Query()["rule_group"],
-		File:      req.URL.Query()["file"],
+		Filter:        AnyRule,
+		RuleName:      req.URL.Query()["rule_name"],
+		RuleGroup:     req.URL.Query()["rule_group"],
+		File:          req.URL.Query()["file"],
+		ExcludeAlerts: excludeAlerts,
+		NextToken:     req.URL.Query().Get("group_next_token"),
+		MaxGroups:     int32(maxGroups),
 	}
 
 	ruleTypeFilter := strings.ToLower(req.URL.Query().Get("type"))
@@ -187,7 +205,7 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 	}
 
 	w.Header().Set("Content-Type", "application/json")
-	rgs, err := a.ruler.GetRules(ctx, rulesReq)
+	rgs, token, err := a.ruler.GetRules(ctx, rulesReq)
 
 	if err != nil {
 		respondServerError(logger, w, err.Error())
@@ -195,7 +213,6 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 	}
 
 	groups := make([]*RuleGroup, 0, len(rgs))
-
 	for _, g := range rgs {
 		grp := RuleGroup{
 			Name:           g.Group.Name,
@@ -209,9 +226,12 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 
 		for i, rl := range g.ActiveRules {
 			if g.ActiveRules[i].Rule.Alert != "" {
-				alerts := make([]*Alert, 0, len(rl.Alerts))
-				for _, a := range rl.Alerts {
-					alerts = append(alerts, alertStateDescToPrometheusAlert(a))
+				var alerts []*Alert
+				if !excludeAlerts {
+					alerts = make([]*Alert, 0, len(rl.Alerts))
+					for _, a := range rl.Alerts {
+						alerts = append(alerts, alertStateDescToPrometheusAlert(a))
+					}
 				}
 				grp.Rules[i] = alertingRule{
 					State:          rl.GetState(),
@@ -241,17 +261,13 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 				}
 			}
 		}
+
 		groups = append(groups, &grp)
 	}
 
-	// keep data.groups are in order
-	sort.Slice(groups, func(i, j int) bool {
-		return groups[i].File < groups[j].File
-	})
-
 	b, err := json.Marshal(&response{
 		Status: "success",
-		Data:   &RuleDiscovery{RuleGroups: groups},
+		Data:   &RuleDiscovery{RuleGroups: groups, NextToken: token},
 	})
 	if err != nil {
 		level.Error(logger).Log("msg", "error marshaling json response", "err", err)
@@ -265,6 +281,20 @@ func (a *API) PrometheusRules(w http.ResponseWriter, req *http.Request) {
 	}
 }
 
+func parseExcludeAlerts(req *http.Request) (bool, error) {
+	excludeAlerts := req.URL.Query().Get("exclude_alerts")
+	if excludeAlerts == "" {
+		return false, nil
+	}
+
+	value, err := strconv.ParseBool(excludeAlerts)
+	if err != nil {
+		return false, fmt.Errorf("unable to parse exclude_alerts value %w", err)
+	}
+
+	return value, nil
+}
+
 func (a *API) PrometheusAlerts(w http.ResponseWriter, req *http.Request) {
 	logger, ctx := spanlogger.NewWithLogger(req.Context(), a.logger, "API.PrometheusAlerts")
 	defer logger.Finish()
@@ -277,7 +307,7 @@ func (a *API) PrometheusAlerts(w http.ResponseWriter, req *http.Request) {
 	}
 
 	w.Header().Set("Content-Type", "application/json")
-	rgs, err := a.ruler.GetRules(ctx, RulesRequest{Filter: AlertingRule})
+	rgs, _, err := a.ruler.GetRules(ctx, RulesRequest{Filter: AlertingRule})
 
 	if err != nil {
 		respondServerError(logger, w, err.Error())
@@ -448,7 +478,9 @@ func (a *API) ListRules(w http.ResponseWriter, req *http.Request) {
 	}
 
 	level.Debug(logger).Log("msg", "retrieving rule groups with namespace", "userID", userID, "namespace", namespace)
-	rgs, err := a.store.ListRuleGroupsForUserAndNamespace(ctx, userID, namespace)
+	// Disable any caching when getting list of all rule groups since listing results
+	// are cached and not invalidated and this API is expected to be strongly consistent.
+	rgs, err := a.store.ListRuleGroupsForUserAndNamespace(ctx, userID, namespace, rulestore.WithCacheDisabled())
 	if err != nil {
 		http.Error(w, err.Error(), http.StatusBadRequest)
 		return
@@ -468,9 +500,41 @@ func (a *API) ListRules(w http.ResponseWriter, req *http.Request) {
 		return
 	}
 	if len(missing) > 0 {
-		// This API is expected to be strongly consistent, so it's an error if any rule group was missing.
-		http.Error(w, fmt.Sprintf("an error occurred while loading %d rule groups", len(missing)), http.StatusInternalServerError)
-		return
+		// This API is expected to be strongly consistent, but we expect the object storage to be strongly
+		// consistent too. This means that if a rule group existed when we listed the storage but doesn't exist
+		// (it's missing) when we load it, then it could have been deleted in the meanwhile.
+		//
+		// We don't want to consider this condition a failure, so we don't return an error in this case, but we
+		// just log a warning because it could be of interest for an operator to further investigate it.
+		level.Warn(logger).Log(
+			"msg", "list rules API skipped some rule groups, because missing when loading them after listing the storage (this could be due to rule groups deleted between listing the storage and getting rule groups content)",
+			"user", userID,
+			"listed_rule_groups", len(rgs),
+			"missing_rule_groups", len(missing),
+			// Logging all rule groups may excessive, but logging at least 1 may give some hints.
+			"first_missing_rule_group_namespace", missing[0].Namespace,
+			"first_missing_rule_group_name", missing[0].Name)
+
+		// Filter out missing rule groups, so they're not returned by the API (they haven't been loaded,
+		// so their content is empty).
+		numRuleGroupsBeforeFiltering := len(rgs)
+		tenantRuleGroups := map[string]rulespb.RuleGroupList{userID: rgs}
+		tenantRuleGroups = filterRuleGroupsByNotMissing(tenantRuleGroups, missing, a.logger)
+
+		var tenantFound bool
+		rgs, tenantFound = tenantRuleGroups[userID]
+
+		if !tenantFound && len(missing) < len(rgs) {
+			// This should never happen, unless a bug.
+			level.Error(logger).Log(
+				"msg", "list rules API has filtered out more rule groups than expected when removing missing rule groups from the response",
+				"user", userID,
+				"rule_groups_before_filtering", numRuleGroupsBeforeFiltering,
+				"rule_groups_to_filter", len(missing))
+
+			http.Error(w, "an error occurred when filtering missing rule groups", http.StatusInternalServerError)
+			return
+		}
 	}
 
 	numRules := 0
@@ -581,7 +645,9 @@ func (a *API) CreateRuleGroup(w http.ResponseWriter, req *http.Request) {
 
 	// Only list rule groups when enforcing a max number of groups for this tenant and namespace.
 	if a.ruler.IsMaxRuleGroupsLimited(userID, namespace) {
-		rgs, err := a.store.ListRuleGroupsForUserAndNamespace(ctx, userID, "")
+		// Disable any caching when getting list of all rule groups since listing results
+		// are cached and not invalidated and we need the most up-to-date number.
+		rgs, err := a.store.ListRuleGroupsForUserAndNamespace(ctx, userID, "", rulestore.WithCacheDisabled())
 		if err != nil {
 			level.Error(logger).Log("msg", "unable to fetch current rule groups for validation", "err", err.Error(), "user", userID)
 			http.Error(w, err.Error(), http.StatusInternalServerError)
