@@ -17,6 +17,8 @@ import (
 	"github.com/prometheus/client_golang/prometheus/promauto"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/grafana/mimir/pkg/scheduler/queue/tree"
 )
 
 const querierForgetDelay = 0
@@ -375,39 +377,36 @@ func TestMultiDimensionalQueueAlgorithmSlowConsumerEffects(t *testing.T) {
 	for _, weightedQueueDimensionTestCase := range weightedQueueDimensionTestCases {
 		numTenants := len(weightedQueueDimensionTestCase.tenantQueueDimensionsWeights)
 
-		tqa := newTenantQuerierAssignments()
+		tqaFlipped := tree.NewTenantQuerierQueuingAlgorithm()
+		tqaQuerierWorkerPrioritization := tree.NewTenantQuerierQueuingAlgorithm()
 
-		nonFlippedRoundRobinTree, err := NewTree(tqa, &roundRobinState{}, &roundRobinState{})
+		flippedRoundRobinTree, err := tree.NewTree(tree.NewRoundRobinState(), tqaFlipped)
 		require.NoError(t, err)
 
-		flippedRoundRobinTree, err := NewTree(&roundRobinState{}, tqa, &roundRobinState{})
+		querierWorkerPrioritizationTree, err := tree.NewTree(tree.NewQuerierWorkerQueuePriorityAlgo(), tqaQuerierWorkerPrioritization)
 		require.NoError(t, err)
 
-		querierWorkerPrioritizationTree, err := NewTree(NewQuerierWorkerQueuePriorityAlgo(), tqa, &roundRobinState{})
-		require.NoError(t, err)
-
-		trees := []struct {
+		treeScenarios := []struct {
 			name string
-			tree Tree
+			tree tree.Tree
+			tqa  *tree.TenantQuerierQueuingAlgorithm
 		}{
 			// keeping these names the same length keeps logged results aligned
-			{
-				"tenant-querier -> query component round-robin tree",
-				nonFlippedRoundRobinTree,
-			},
 			{
 				"query component round-robin -> tenant-querier tree",
 				flippedRoundRobinTree,
+				tqaFlipped,
 			},
 			{
 				"worker-queue prioritization -> tenant-querier tree",
 				querierWorkerPrioritizationTree,
+				tqaQuerierWorkerPrioritization,
 			},
 		}
-		for _, tree := range trees {
+		for _, scenario := range treeScenarios {
 			testCaseName := fmt.Sprintf(
 				"tree: %s, %s",
-				tree.name,
+				scenario.name,
 				weightedQueueDimensionTestCase.name,
 			)
 			testCaseObservations := &testScenarioQueueDurationObservations{
@@ -416,14 +415,10 @@ func TestMultiDimensionalQueueAlgorithmSlowConsumerEffects(t *testing.T) {
 				queryComponentQueueDurationObservations: map[string][]float64{},
 			}
 
-			// only the non-flipped tree uses the old tenant -> query component hierarchy
-			prioritizeQueryComponents := tree.tree != nonFlippedRoundRobinTree
-
 			t.Run(testCaseName, func(t *testing.T) {
 				queue, err := NewRequestQueue(
 					log.NewNopLogger(),
 					maxOutStandingPerTenant,
-					prioritizeQueryComponents,
 					querierForgetDelay,
 					promauto.With(nil).NewGaugeVec(prometheus.GaugeOpts{}, []string{"user"}),
 					promauto.With(nil).NewCounterVec(prometheus.CounterOpts{}, []string{"user"}),
@@ -432,17 +427,17 @@ func TestMultiDimensionalQueueAlgorithmSlowConsumerEffects(t *testing.T) {
 				)
 				require.NoError(t, err)
 
-				// NewRequestQueue constructor does not allow passing in a tree or tenantQuerierAssignments
+				// NewRequestQueue constructor does not allow passing in a tree or tenantQuerierShards
 				// so we have to override here to use the same structures as the test case
-				queue.queueBroker.tenantQuerierAssignments = tqa
-				queue.queueBroker.prioritizeQueryComponents = prioritizeQueryComponents
-				queue.queueBroker.tree = tree.tree
+				queue.queueBroker.tenantQuerierAssignments = &tenantQuerierShards{
+					querierIDsSorted: make([]tree.QuerierID, 0),
+					tenantsByID:      make(map[string]*queueTenant),
+					queuingAlgorithm: scenario.tqa,
+				}
+				queue.queueBroker.tree = scenario.tree
 
 				ctx := context.Background()
 				require.NoError(t, queue.starting(ctx))
-				t.Cleanup(func() {
-					require.NoError(t, queue.stop(nil))
-				})
 
 				// configure queue producers to enqueue requests with the query component
 				// randomly assigned according to the distribution defined in the test case
@@ -481,10 +476,14 @@ func TestMultiDimensionalQueueAlgorithmSlowConsumerEffects(t *testing.T) {
 				testCaseNames = append(testCaseNames, testCaseName)
 				testCaseReports[testCaseName] = report
 
-				// ensure everything was dequeued
-				path, val := tree.tree.Dequeue(&DequeueArgs{querierID: tqa.currentQuerier})
+				require.NoError(t, queue.stop(nil))
+				assert.NotEqual(t, "", tree.CurrentQuerier(scenario.tqa))
+				// ensure everything was dequeued; we can pass a nil DequeueArgs because we don't
+				// want to update any state before doing this (i.e., we're dequeuing for _any_ querier,
+				// just to make sure the tree is empty).
+				path, val := scenario.tree.Dequeue(nil)
 				assert.Nil(t, val)
-				assert.Equal(t, path, QueuePath{})
+				assert.Equal(t, path, tree.QueuePath{})
 			})
 		}
 	}
